#!/usr/bin/tcl -f

################################################
#               FASTA parser                   #
# Parser extracts description line, identity,  #
# expectation and direction of translation     #
# frame from results of FASTA(N)(P)(X) search  #
#                                              #
# Alexander Kozik, Copyright, December 15 2001 #
#                                              #
################################################

# FASTA-P Procedure
proc FastaP_Parser_Subject {f_out current_line query_id description_switch frame_info} {

    global subject_found delimiter_type just_an_id

    set empty_string ""
    set subject_pattern {^\>\>}
    set score_pattern {Z-score}
    set expect_pattern {E()}
    set identity_pattern {\% identity}
    set overlap_pattern {aa overlap}
    if {[regexp "$subject_pattern" "$current_line"] && $just_an_id == "YES"} {
	regsub $subject_pattern $current_line $empty_string current_description
	regsub -all {\s+} $current_description " " current_description
	regsub {^gi\|} $current_description $empty_string current_description
	########
	if {$delimiter_type == "T"} {
	    regsub {\|} $current_description "\t" current_description
	}
	if {$delimiter_type == "S"} {
	    regsub { } $current_description "\t" current_description
	}
	########
	if {$description_switch == "N" || $description_switch == "n"} {
	    regsub {\s.*} $current_description $empty_string current_description
	    regsub {\|.*} $current_description $empty_string current_description
	}
	puts -nonewline $f_out "$query_id\t$current_description\t"
    }
    if {[regexp "$subject_pattern" "$current_line"] && $just_an_id == "NO"} {
        regsub $subject_pattern $current_line $empty_string current_description
        regsub -all { } $current_description "" current_description
        regsub -all {\(.*\)} $current_description "" current_description
        regsub {^gi\|} $current_description $empty_string current_description
        puts -nonewline $f_out "$query_id\t$current_description\t"
    }
    if {[regexp "$score_pattern" "$current_line"] && [regexp "$expect_pattern" "$current_line"] && $subject_found == "YES"} {
	regsub {initn.*E\(\)\:} $current_line $empty_string current_exp
	regsub -all { } $current_exp $empty_string current_exp
	if {[regexp {rev-comp} $current_exp] && $frame_info == "YES"} {
	    set strand_direction "REV"
	} else {
	    set strand_direction "FOR"
	}
	regsub -all {rev-comp} $current_exp $empty_string current_exp
	if {$frame_info == "YES"} {
	    puts -nonewline $f_out "$strand_direction\t$current_exp\t"
	} else {
	    puts -nonewline $f_out "$current_exp\t"
	}
    }
    if {[regexp "$identity_pattern" "$current_line"] && [regexp "$overlap_pattern" "$current_line"] && $subject_found == "YES"} {
	set current_sim $current_line
	regsub {\%\sidentity.*} $current_sim $empty_string current_sim
	regsub {Smith-Waterman.*\;} $current_sim $empty_string current_sim
	regsub -all { } $current_sim $empty_string current_sim
	puts $f_out "$current_sim"
	set subject_found "NO"
    }
}

# FASTA-X Procedure
proc FastaX_Parser_Subject {f_out current_line query_id description_switch frame_info} {

    global subject_found delimiter_type just_an_id

    set empty_string ""
    set subject_pattern {^\>\>}
    set score_pattern {Z-score}
    set expect_pattern {E()}
    set identity_pattern {\% identity}
    set overlap_pattern {aa overlap}
    if {[regexp "$subject_pattern" "$current_line"] && $just_an_id == "YES"} {
	regsub $subject_pattern $current_line $empty_string current_description
	regsub -all {\s+} $current_description " " current_description
	regsub {^gi\|} $current_description $empty_string current_description
	########
	if {$delimiter_type == "T"} {
	    regsub {\|} $current_description "\t" current_description
	}
	if {$delimiter_type == "S"} {
	    regsub { } $current_description "\t" current_description
	}
	########
	if {$description_switch == "N" || $description_switch == "n"} {
	    regsub {\s.*} $current_description $empty_string current_description
	    regsub {\|.*} $current_description $empty_string current_description
	}
	puts -nonewline $f_out "$query_id\t$current_description\t"
    }
    if {[regexp "$subject_pattern" "$current_line"] && $just_an_id == "NO"} {
        regsub $subject_pattern $current_line $empty_string current_description
        regsub -all { } $current_description "" current_description
        regsub -all {\(.*\)} $current_description "" current_description
	regsub {^gi\|} $current_description $empty_string current_description
        puts -nonewline $f_out "$query_id\t$current_description\t"
    }
    if {[regexp "$score_pattern" "$current_line"] && [regexp "$expect_pattern" "$current_line"] && $subject_found == "YES"} {
	regsub {initn.*E\(\)\:} $current_line $empty_string current_exp
	regsub -all { } $current_exp $empty_string current_exp
	if {[regexp {rev-comp} $current_exp] && $frame_info == "YES"} {
	    set strand_direction "REV"
	} else {
	    set strand_direction "FOR"
	}
	regsub -all {rev-comp} $current_exp $empty_string current_exp
	if {$frame_info == "YES"} {
	    puts -nonewline $f_out "$strand_direction\t$current_exp\t"
	} else {
	    puts -nonewline $f_out "$current_exp\t"
	}
    }
    if {[regexp "$identity_pattern" "$current_line"] && [regexp "$overlap_pattern" "$current_line"] && $subject_found == "YES"} {
	set current_sim $current_line
	regsub {\%\sidentity.*} $current_sim $empty_string current_sim
	regsub {Smith-Waterman.*\;} $current_sim $empty_string current_sim
	regsub -all { } $current_sim $empty_string current_sim
	puts $f_out "$current_sim"
	set subject_found "NO"
    }
}

# FASTA-N Procedure
proc FastaN_Parser_Subject {f_out current_line query_id description_switch strand_info} {

    global subject_found delimiter_type just_an_id

    set empty_string ""
    set subject_pattern {^\>\>}
    set score_pattern {Z-score}
    set expect_pattern {E()}
    set identity_pattern {\% identity}
    set overlap_pattern {nt overlap}
    if {[regexp "$subject_pattern" "$current_line"] && $just_an_id == "YES"} {
	regsub $subject_pattern $current_line $empty_string current_description
	regsub -all {\s+} $current_description " " current_description
	regsub {^gi\|} $current_description $empty_string current_description
	########
	if {$delimiter_type == "T"} {
	    regsub {\|} $current_description "\t" current_description
	}
	if {$delimiter_type == "S"} {
	    regsub { } $current_description "\t" current_description
	}
	########
	if {$description_switch == "N" || $description_switch == "n"} {
	    regsub {\s.*} $current_description $empty_string current_description
	    regsub {\|.*} $current_description $empty_string current_description
	}
	puts -nonewline $f_out "$query_id\t$current_description\t"
    }
    if {[regexp "$subject_pattern" "$current_line"] && $just_an_id == "NO"} {
	regsub $subject_pattern $current_line $empty_string current_description
	regsub -all { } $current_description "" current_description
	regsub -all {\(.*\)} $current_description "" current_description
	regsub {^gi\|} $current_description $empty_string current_description
	puts -nonewline $f_out "$query_id\t$current_description\t"
    }
    if {[regexp "$score_pattern" "$current_line"] && [regexp "$expect_pattern" "$current_line"] && $subject_found == "YES"} {
	regsub {initn.*E\(\)\:} $current_line $empty_string current_exp
	regsub -all { } $current_exp $empty_string current_exp
	if {[regexp {rev-comp} $current_exp] && $strand_info == "YES"} {
	    set strand_direction "REV"
	} else {
	    set strand_direction "FOR"
	}
	regsub -all {rev-comp} $current_exp $empty_string current_exp
	if {$strand_info == "YES"} {
	    puts -nonewline $f_out "$strand_direction\t$current_exp\t"
	} else {
	    puts -nonewline $f_out "$current_exp\t"
	}
    }
    if {[regexp "$identity_pattern" "$current_line"] && [regexp "$overlap_pattern" "$current_line"] && $subject_found == "YES"} {
	set current_sim $current_line
	regsub {\%\sidentity.*} $current_sim $empty_string current_sim
	regsub {banded Smith-Waterman score.*\;} $current_sim $empty_string current_sim
	regsub -all { } $current_sim $empty_string current_sim
	puts $f_out "$current_sim"
	set subject_found "NO"
    }
}

#################################
#         MAIN BODY             #
#################################

# standard input/output dialog #

puts -nonewline "FASTA program was \"(m)pi\" or \"(r)egular\": "
flush stdout

set fasta_program_type [gets stdin]

if {$fasta_program_type == "m" || $fasta_program_type == "M"} {
    set fasta_program_type "MPI"
}

if {$fasta_program_type == "r" || $fasta_program_type == "R"} {
    set fasta_program_type "REGULAR"
}

puts -nonewline "Enter the SOURCE file name: "
flush stdout

set sourcefile [gets stdin]

while {[catch {set f_in [open "$sourcefile"  "r"]}] == 1} {
        puts stderr "Error opening $sourcefile"
        puts -nonewline "Please re-enter the SOURCE file name: "
        flush stdout
        set sourcefile [gets stdin]
}

puts -nonewline "Enter the DESTINATION file name: "
flush stdout

set destfile [gets stdin]

set f_out [open "$destfile"  "w"]

global just_an_id

puts -nonewline "Does SOURCE contain description line? (y/n): "
flush stdout

set just_an_id [gets stdin]

if {$just_an_id == "y" || $just_an_id == "Y"} {
    set just_an_id "YES"
    set description_switch "WHATEVER"
}
if {$just_an_id == "n" || $just_an_id == "N"} {
    set just_an_id "NO"
    set description_switch "WHATEVER"
}

if {$just_an_id == "YES"} {
    puts -nonewline "extract DESCRIPTION line (Y/N): "
    flush stdout
    set description_switch [gets stdin]

    puts -nonewline "Subject DELIMITER type \"|\"(t) or \" \"(s): "
    flush stdout
    set delimiter_type [gets stdin]

    if {$delimiter_type == "t" || $delimiter_type == "T"} {
	set delimiter_type "T"
    }
    if {$delimiter_type == "s" || $delimiter_type == "S"} {
	set delimiter_type "S"
    }
}

puts -nonewline "type of FASTA search was fasta(n) fasta(x) fasta(p): "
flush stdout

set type_of_fasta [gets stdin]

if {$type_of_fasta == "P" || $type_of_fasta == "p"} {
    set frame_info "NO"
}

if {$type_of_fasta == "X" || $type_of_fasta == "x"} {
    puts -nonewline "Extract translation frame info (y)es/(n)o: "
    flush stdout
    set frame_info [gets stdin]
    if {$frame_info == "Y" || $frame_info == "y"} {
	set frame_info "YES"
    }
}

if {$type_of_fasta == "N" || $type_of_fasta == "n"} {
    puts -nonewline "Extract strand direction info (y)es/(n)o: "
    flush stdout
    set strand_info [gets stdin]
    if {$strand_info == "Y" || $strand_info == "y"} {
	set strand_info "YES"
    }
}

# end of input/output dialog #

global subject_found delimiter_type

# set query_pattern_mpi {\s\s1\>\>}
set query_pattern_mpi {\s\s1\>\>\>}
set query_pattern_reg {^\s\>}
set subject_pattern {^\>\>}
set fasta_header {>}
set new_line "\n"
set empty_string ""
set atgc_seq {^(A|B|C|D|E|F|G|H|I|J|K|L|M|N|O|P|Q|R|S|T|U|V|W|X|Y|Z|a|b|c|d|e|f|g|h|i|j|k|l|m|n|o|p|q|r|s|t|u|v|w|x|y|z)}
set alignment_flag "OFF"

set lineNumber 0
set first_line_to_process 0

while {[gets $f_in  current_line] >= 0}  {

        if {$lineNumber >= $first_line_to_process} {
	    if {[regexp "$query_pattern_reg" "$current_line"] && $fasta_program_type == "REGULAR"} {
		regsub $query_pattern_reg $current_line $empty_string query_id
		regsub {\s.*} $query_id $empty_string query_id
		regsub {\|.*} $query_id $empty_string query_id
		regsub -all {\,} $query_id $empty_string query_id
		if {$description_switch == "Y" || $description_switch == "y"} {
		    puts $f_out "*** BEST HITS FOR: $query_id\ ***"
		}
	    }
	    if {[regexp "$query_pattern_reg" "$current_line"] && $fasta_program_type == "REGULAR"} {
		set alignment_flag "ON"
	    }
	    ###################################################################################
	    if {[regexp "$query_pattern_mpi" "$current_line"] && $fasta_program_type == "MPI"} {
		regsub $query_pattern_mpi $current_line $empty_string query_id
		regsub {\s.*} $query_id $empty_string query_id
		regsub {\|.*} $query_id $empty_string query_id
		regsub -all {\,} $query_id $empty_string query_id
		if {$description_switch == "Y" || $description_switch == "y"} {
		    puts $f_out "*** BEST HITS FOR: $query_id\ ***"
		}
	    }
	    if {[regexp "$query_pattern_mpi" "$current_line"] && $fasta_program_type == "MPI"} {
		set alignment_flag "ON"
	    }
	    if [regexp "$subject_pattern" "$current_line"] {
		set subject_found "YES"
#		puts $f_out $current_line
	    }
	    if {$alignment_flag == "ON"} {
		if {$type_of_fasta == "X" || $type_of_fasta == "x"} {
		    FastaX_Parser_Subject $f_out $current_line $query_id \
			    $description_switch $frame_info
		}
		if {$type_of_fasta == "P" || $type_of_fasta == "p"} {
		    FastaP_Parser_Subject $f_out $current_line $query_id \
			    $description_switch $frame_info
		}
		if {$type_of_fasta == "N" || $type_of_fasta == "n"} {
		    FastaN_Parser_Subject $f_out $current_line $query_id \
			    $description_switch $strand_info
		}
	    }
	    if [regexp "No sequences with E()" $current_line] {
		puts $f_out "$query_id\tNo significant similarity found"
	    }
	}
	incr lineNumber
    }
#    puts -nonewline $f_out $new_line
close $f_in
close $f_out
